from django.contrib import admin
from .models import SmartCache


@admin.register(SmartCache)
class SmartCacheAdmin(admin.ModelAdmin):
    list_display = (
        'key',
    )
    readonly_fields = (
        'key',
        'value'
    )

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False



# Generated by Django 2.2.3 on 2020-07-23 11:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_smart_redis', '0002_smartcache_content_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='smartcache',
            name='qs',
            field=models.TextField(default='', verbose_name='Query String'),
            preserve_default=False,
        ),
    ]



import json
import redis
import logging

from ..settings import redis_connect, namespace

logger = logging.getLogger(__name__)


class EzRedis(redis.StrictRedis):
    def __init__(self, *args, **kwargs):
        self.namespace = kwargs.pop('namespace') or ''
        super().__init__(*args, **kwargs)

    def __get_name(self, key):
        return f'{self.namespace}:{key}' if self.namespace else f'{key}'

    def ez_set_json(self, key: str, data) -> bool:
        return self.set(self.__get_name(key), json.dumps(data))

    def ez_set(self, key: str, data: dict) -> bool:
        return self.set(self.__get_name(key), data)

    def set(self, name, value, *args, **kwargs) -> bool:
        return super().set(self.__get_name(name), value, *args, **kwargs)

    def ez_get(self, key: str) -> str:
        return self.get(key).decode()

    def ez_get_json(self, key: str) -> dict:
        return json.loads(self.ez_get(key))

    def exists(self, key: str):
        return super().exists(self.__get_name(key))

    def ez_delete(self, key: str):
        return super().delete(self.__get_name(key))

    def get(self, key: str):
        return super().get(self.__get_name(key))


def clear_cache(key: (str, list, tuple)):
    def _clear(_key):
        try:
            with EzRedis(**redis_connect) as r:
                logger.debug(f"Delete cache '{_key}'")
                for __key in r.scan_iter(f"{namespace}:{_key}*"):
                    logger.debug(f">>> Delete key '{__key.decode()}'")
                    r.delete(__key)

        except redis.exceptions.ConnectionError as e:
            logger.error(e)

    if isinstance(key, (list, tuple)):
        for item in key:
            _clear(item)
    else:
        _clear(key)



import functools
import logging

import redis
import hashlib
from django.http import HttpResponse
from django.template.response import ContentNotRenderedError

from .core import EzRedis
from ..models import SmartCache
from ..settings import redis_connect

logger = logging.getLogger(__name__)


def get_key_hash(key, qs_hash) -> str:
    if qs_hash:
        return key + ':' + qs_hash
    return key


def get_content_type(response) -> str:
    headers = getattr(response, '_headers', {})
    return headers.get('content-type')[1] if headers.get('content-type') else ''


def smart_cache(key):
    def decorate(method):
        @functools.wraps(method)
        def wrapper(self, *args, **kwargs):
            qs_hash = ''
            qs = self.request.META.get('QUERY_STRING')
            if self.request.META.get('QUERY_STRING'):
                qs_hash = hashlib.sha512(qs.encode()).hexdigest()

            try:
                with EzRedis(**redis_connect) as r:
                    if int(r.exists(get_key_hash(key, qs_hash))):
                        return HttpResponse(
                            r.get(get_key_hash(key, qs_hash)),
                            content_type=r.get(key + ':content_type')
                        )

                    response = method(self, *args, **kwargs)

                    try:
                        content = response.content.decode()
                    except ContentNotRenderedError:
                        content = response.rendered_content

                    r.set(get_key_hash(key, qs_hash), content)
                    r.set(key + ':content_type', get_content_type(response))

                    SmartCache._default_manager.create(
                        key=get_key_hash(key, qs_hash),
                        value=content,
                        content_type=get_content_type(response),
                        qs=qs,
                    )

            except redis.exceptions.ConnectionError:
                logger.error("!!! Cache not active !!!")
                return method(self, *args, **kwargs)

            return response

        return wrapper

    return decorate



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-smart-redis',
    version='1.5.2',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License (MIT)',
    description='A library for easy use redis.',
    long_description=README,
    install_requires=['redis==3.5.*', ],
    url='https://github.com/MrEarnestWhim/smart-redis',
    author='Mr Earnest Whim',
    author_email='5494567@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
